import fullscreen.*; 
FullScreen fs;
HandCapture   handCapture;

PVector[] matrix;
int stepWidth;
int countHorizontal;
int countVertical;
int countTotal;
float x;
float y;
float z;
float targetX, targetY, targetZ;

int setSize        = 30;
int setSpace       = 15;
int setMargin      = 20;
int setSign        = 1;
color setBackground  = color(255);
float setEasing    = 0.3;
boolean setZoom    = false;

void setup(){
  
  size(1024,768);
  frameRate(20);
  background(255);
  noStroke();
  smooth();
  noCursor();
  
  handCapture = new HandCapture(this);
  
  int iWidth = width - setMargin;
  int iHeight = height - setMargin;
  
  fs = new FullScreen(this); 
  fs.enter(); 
  
  stepWidth = setSize + setSpace;
  countHorizontal = iWidth / stepWidth ;
  countVertical = iHeight / stepWidth;
  countTotal = countHorizontal * countVertical;
    
  matrix = new PVector[countTotal];
  int i = 0;
  for(int y = stepWidth; y <= iHeight; y += stepWidth) {
    for(int x = stepWidth; x <= iWidth; x += stepWidth) {
      matrix[i] = new PVector(x, y, 0);
      i++;
    }
  }

}

void draw(){
  
  handCapture.update();
  
  fill(setBackground, 200);
  rect(0,0,width,height);
  
  PVector kinectVect = handCapture.handPos();
  
  // easing
  targetX = map(kinectVect.x,0,640,0,width);
  float dx = targetX - x;
  if(abs(dx) > 1) {
    x += dx * setEasing;
  }
  
  targetY = map(kinectVect.y,0,480,0,height);
  float dy = targetY - y;
  if(abs(dy) > 1) {
    y += dy * setEasing;
  }
  
  PVector mouseVect = new PVector(x, y, 0);
  
  for(int i = 0; i < countTotal; i++) {
    float distValue = PVector.dist(mouseVect, matrix[i]);
    float angleMouse = atan2(mouseVect.y - matrix[i].y, mouseVect.x - matrix[i].x);
    float state = map(distValue, 0, width*1.41-70, 1, 7);
    
    pushMatrix();
    
      if(setZoom) {
        
        targetZ = map(kinectVect.z,200,5000,5,1);
        float dz = targetZ - z;
        if(abs(dz) > 1) {
          z += dz * setEasing;
      }
        
        float factor = map(kinectVect.z,200,5000,1,5);
        translate(map(factor,1,5,0,width*-1),map(factor,1,5,0,height*-1));
        scale(factor);
      }
      
      pushMatrix();  
        translate(matrix[i].x, matrix[i].y);
          switch(setSign) {
            case 1:
              sign1(state, angleMouse);
              break;
            case 2:
              sign2(state, angleMouse);
              break;
              case 3:
              sign3(state, angleMouse);
              break;
            case 4:
              sign4(state, angleMouse);
              break;
          }
      popMatrix();
      
    popMatrix();
    
  }

}

void keyPressed () {
  
 switch(key) { 
   case '1':
     setSign = 1;
     setBackground  = color(255);
     break;
     
   case '2':
     setSign = 2;
     setBackground  = color(255);
     break;
     
   case '3':
     setSign = 3;
     setBackground  = color(255);
     break;
     
   case '4':
     setSign = 4;
     setBackground  = color(0);
     break;
     
   case 'z':
     if(setZoom)
       setZoom = false;
      else
        setZoom = true;
     break;
 }
  
}

void sign1(float s, float a){

  pushMatrix();
  
    rotate(a);
    scale(map(s,1,7,1.4,0));

    fill(0);
    ellipse(0,0,setSize,setSize);
    
    fill(255);
    arc(0, 0, setSize+5, setSize+5, 0, map(a,1,7,PI/2,PI));
    
   popMatrix();
   
}

void sign2(float s, float a){

  pushMatrix();
    rotate(a);
    //scale(map(s,1,7,1.5,-0.5));    

    fill(0);
    rotate(radians(90));
    //rotate(radians(random(-3,3)));
    scale(0.6);
    beginShape();
      vertex(0,setSize/-2);
      vertex(setSize/2.5,setSize/2);
      vertex(setSize/-2.5,setSize/2);
    endShape(CLOSE);
    
   popMatrix();
   
}

void sign3(float s, float a){

  pushMatrix();
    rotate(a);
    scale(map(s,1,7,1.5,-0.5));    

    fill(0);
    rotate(radians(90));
    //rotate(radians(random(-3,3)));
    beginShape();
      vertex(0,setSize/-2);
      vertex(setSize/2.5,setSize/2);
      vertex(setSize/-2.5,setSize/2);
    endShape(CLOSE);
    
   popMatrix();
   
}

void sign4(float s, float a){  
    
  pushMatrix();
    rotate(a);
    scale(map(s,1,7,0,4.5));    
  
    fill(255);
    rotate(radians(90));
    //rotate(radians(random(-3,3)));
    beginShape();
      vertex(0,setSize/-2);
      vertex(setSize/2.5,setSize/2);
      vertex(setSize/-2.5,setSize/2);
    endShape(CLOSE);
   
   popMatrix();
       
}

void onCreateHands(int handId,PVector pos,float time) {
  handCapture.onCreateHands(handId,pos,time);
}

void onUpdateHands(int handId,PVector pos,float time) {
  handCapture.onUpdateHands(handId,pos,time);
}

void onDestroyHands(int handId,float time) {
  handCapture.onDestroyHands(handId,time);
}

void onRecognizeGesture(String strGesture, PVector idPosition, PVector endPosition) {
  handCapture.onRecognizeGesture(strGesture,idPosition,endPosition);
}

void onProgressGesture(String strGesture, PVector position,float progress) {
  handCapture.onProgressGesture(strGesture,position,progress);
}
